
html, body {
  padding: 0;
  margin: 0;
  min-height: 100%;
  height: 100%;
  width: 100%;
}

#GB_overlay {
  position: absolute;
  z-index:100;
  top: 0px;
  left: 0px;
  background-color:#000;
  filter:alpha(opacity=60);
  -moz-opacity: 0.6;
  opacity: 0.6;
}

#GB_window {
  top: 50%;
  left: 50%;
  position: fixed;
  background: #fff;
  border: 5px solid #aaa;
  overflow: auto;
  width: 400px;
  height: 400px;
  z-index: 150;
}

#GB_frame {
  border: 0;
  overflow: auto;
  width: 100%;
  height: 378px;
}

#GB_caption {
  font: 12px bold helvetica, verdana, sans-serif;
  color: #fff;
  background: #888;
  padding: 2px 0 2px 5px;
  margin: 0;
  text-align: left;
}

#GB_window img {
  position: absolute;
  top: 2px;
  right: 5px;
  cursor: pointer;
  cursor: hand;
}

